@@ -59,6 +59,7 @@
 #include <xorg-config.h>
 #endif
 
+#include "os.h"
 #include "xf86Parser.h"
 #include "xf86tokens.h"
 #include "Configint.h"
@@ -102,8 +103,10 @@ xf86parseInputSection (void)
 		case DRIVER:
 			if (xf86getSubToken (&(ptr->inp_comment)) != STRING)
 				Error (QUOTE_MSG, "Driver");
-                        if (strcmp(val.str, "keyboard") == 0)
-                            ptr->inp_driver = "kbd";
+                        if (strcmp(val.str, "keyboard") == 0) {
+                            ptr->inp_driver = strdup("kbd");
+                            free(val.str);
+                        }
                         else
 			    ptr->inp_driver = val.str;
 			break;
@@ -172,21 +175,14 @@ xf86validateInput (XF86ConfigPtr p)
 {
 	XF86ConfInputPtr input = p->conf_input_lst;
 
-#if 0 /* Enable this later */
-	if (!input) {
-		xf86validationError ("At least one InputDevice section is required.");
-		return (FALSE);
-	}
-#endif
-
 	while (input) {
 		if (!input->inp_driver) {
 			xf86validationError (UNDEFINED_INPUTDRIVER_MSG, input->inp_identifier);
-			return (FALSE);
+			return FALSE;
 		}
 		input = input->list.next;
 	}
-	return (TRUE);
+	return TRUE;
 }
 
 XF86ConfInputPtr
@@ -195,11 +191,11 @@ xf86findInput (const char *ident, XF86ConfInputPtr p)
 	while (p)
 	{
 		if (xf86nameCompare (ident, p->inp_identifier) == 0)
-			return (p);
+			return p;
 
 		p = p->list.next;
 	}
-	return (NULL);
+	return NULL;
 }
 
 XF86ConfInputPtr
@@ -208,10 +204,10 @@ xf86findInputByDriver (const char *driver, XF86ConfInputPtr p)
 	while (p)
 	{
 		if (xf86nameCompare (driver, p->inp_driver) == 0)
-			return (p);
+			return p;
 
 		p = p->list.next;
 	}
-	return (NULL);
+	return NULL;
 }
 
